︠b7f3b3d4-2f78-44a7-8426-1c66c0766e6cs︠
#----------------------------------------------------------------
# Sample code for UEET2533 Information Theory and Coding
# by Chong Zan Kai
# Email chongzk@utar.edu.my
# Universiti Tunku Abdul Rahman, Malaysia.
#
# We demonstrate how to use repetition code to secure an image
# against a BSC channel.
#
# Last modified on 9-Jan-2015.
#----------------------------------------------------------------

#----------------------------------------------------------------
# Functions to convert image to matrix and vice versa.
#----------------------------------------------------------------
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import numpy as np

#
# Convert image 2darray to GF2 matrix.
#
def img_arr_to_matrix(img_2darr):
    #img_2darr = mpimg.imread(bmp_file)
    img_2dlist = []
    #
    # Convert to 0 and 1 first.
    for p4_arr in img_2darr:
        a_list = [int(p[0]>0) for p in p4_arr]
        img_2dlist.append(a_list)
    #
    img_mat = matrix(GF(2), img_2dlist)
    return img_mat

#
# Convert GF2 matrix back to image 2darr.
#
def img_matrix_to_arr(img_mat):
    img_2darr = []
    for row in img_mat.rows():
        pixel_list = [ np.uint8([int(r) * 255, int(r) * 255, int(r) * 255, 255 ]) for r in row]
        img_2darr.append(pixel_list)
    return np.array(img_2darr)


#
# Introduce noise to the message in BSC channel.
#
def bsc_channel(msg_mat, p_flip):
    new_msg_mat = copy (msg_mat)
    total_row, total_col = new_msg_mat.dimensions()
    for r in range(total_row):
        for c in range(total_col):
            rand_val = random()
            if rand_val <= p_flip:
                new_msg_mat[r, c] += 1

    return new_msg_mat



#
# Repetition Code Encoder
#
def repetition_code_encoder(msg_mat, repetition):
    red_list = []
    for row in msg_mat.rows():
        new_row = []
        for symbol in row:
            new_row.extend([symbol] * (repetition-1))
        red_list.append (new_row)
    #
    red_mat = matrix(GF(2),  red_list)
    return msg_mat.augment(red_mat)



#
# Repetition Code Decoder
#
def repetition_code_decoder(msg_mat, repetition, org_msg_dim):
    new_mat = zero_matrix(GF(2), org_msg_dim[0], org_msg_dim[1])
    org_total_row, org_total_col = org_msg_dim

    for r in range(org_total_row):
        for c in range(org_total_col):
            val_list = [ msg_mat[r,c] ]
            for i in range(repetition -1):
                val = msg_mat[r, org_total_col + c*(repetition-1) + i ]
                val_list.append(val)
            #print 'val_list', val_list
            if val_list.count(0) > val_list.count(1):
                new_mat[r, c] = 0
            else:
                new_mat[r, c] = 1
    #
    return new_mat


#----------------------------------------------------------------

#----------------------------------------------------------------
# Main
#----------------------------------------------------------------

#----------------------------------------------------------------
# Main
#----------------------------------------------------------------

p_flip = 0.1 # Noise probability
sample_file = 'sample01.bmp'
repetition = 3


#
# Orignial image
#
# Read the image file as array
img_arr = mpimg.imread(sample_file)


# Display the image
print 'Original image:'
im = plt.imshow(img_arr)
plt.show()
print

# Convert image array to GF2 matrix
img_mat = img_arr_to_matrix (img_arr)
print

# We need to keep track of the message dimensions for decoding later.
original_msg_dimension = img_mat.dimensions()

#
# Encode with Repetition code.
#
encoded_img_mat = repetition_code_encoder(img_mat, repetition)
# Display the image
print 'Encoded image:'
encoded_img_arr = img_matrix_to_arr(encoded_img_mat)
im = plt.imshow(encoded_img_arr)
plt.show()
print



#
# Introduce noise to the image
#
recv_img_mat = bsc_channel(encoded_img_mat, p_flip)


#
# Present the received image
#

# Display image
print 'Received image after BSC channel:'
recv_img_arr = img_matrix_to_arr (recv_img_mat)
im = plt.imshow(recv_img_arr)
plt.show()
print


#
# Without Decoder, we have ...
#
print 'Without repetition code, we get:'
noise_img_mat = recv_img_mat.matrix_from_columns(range(original_msg_dimension[1]))
noise_img_arr = img_matrix_to_arr (noise_img_mat)
im = plt.imshow(noise_img_arr)
plt.show()
print


#
# Decoding with Repetition code
#
decoded_img_mat = repetition_code_decoder(recv_img_mat, repetition, original_msg_dimension)
# Display image
print 'After decoded with repetition code:'
decoded_img_arr = img_matrix_to_arr (decoded_img_mat)
im = plt.imshow(decoded_img_arr)
plt.show()
print


︡166f37fe-3753-464c-ad4b-0b887ef7535a︡︡{"stdout":"Original image:\n","done":false}︡{"once":false,"done":false,"file":{"show":true,"uuid":"3721aa46-9e5b-4e60-8fdc-b7840ccdfee4","filename":"0c1e086b-07e7-4af9-a914-ac12d5be33e6.svg"}}︡{"stdout":"\n","done":false}︡{"stdout":"\n","done":false}︡{"stdout":"Encoded image:\n","done":false}︡{"once":false,"done":false,"file":{"show":true,"uuid":"bf0fc92b-d961-4bca-a4dd-7733a098bd50","filename":"93371bbb-8957-4bbd-8c12-5ca09cfa7eb4.svg"}}︡{"stdout":"\n","done":false}︡{"stdout":"Received image after BSC channel:\n","done":false}︡{"once":false,"done":false,"file":{"show":true,"uuid":"61becbc5-210a-43b9-9b28-b60cc08048dc","filename":"6b1e2286-f338-475f-8d24-19520e5baa23.svg"}}︡{"stdout":"\n","done":false}︡{"stdout":"Without repetition code, we get:\n","done":false}︡{"once":false,"done":false,"file":{"show":true,"uuid":"d9eb8718-a647-4b5d-b9e4-f2618d35fbbf","filename":"faa90dbb-dc15-4653-9457-d40e8462f08c.svg"}}︡{"stdout":"\n","done":false}︡{"stdout":"After decoded with repetition code:\n","done":false}︡{"once":false,"done":false,"file":{"show":true,"uuid":"157ba98a-cdb8-4fdc-b849-7c414360e45b","filename":"b4428914-771d-450b-9587-9ce378cefb09.svg"}}︡{"stdout":"\n","done":false}︡{"done":true}
︠7060f1f5-4ed5-44b5-a79b-16485e347046︠
︡08e9040d-369b-488f-a4e7-9b2365258481︡
︠ef99b515-be5b-45be-9515-6215874b1a9f︠













